#include <core.p4>
#include <sume_switch.p4>
#include "headers.p4"

/********************************************************************************
 * Header
 */

typedef bit<48> EthAddr_t;
header Ethernet_h {
    EthAddr_t dstAddr;
    EthAddr_t srcAddr;
    bit<16> etherType;
}

struct Parsed_packet {
    Ethernet_h ethernet;
}

// user defined metadata: can be used to share information between
// TopParser, TopPipe, and TopDeparser
struct user_metadata_t {
    bit<8>  unused;
}

// digest_data, MUST be 256 bits -- what is this used for?
struct digest_data_t {
    bit<256> unused;
}


/********************************************************************************
 * Parser
 */

@Xilinx_MaxPacketRegion(1024)
parser TopParser(packet_in packet,
                 out Parsed_packet hdr,
                 out user_metadata_t meta
                 out digest_data_t digest_data,
                 inout sume_metadata_t sume_metadata) {
    state start {
        meta.unused = 0;
        digest_data.unused = 0;

        packet.extract(hdr.ethernet);
        transition select(hdr.ethernet.ethertype) {
            TYPE_IPV6: ipv6;
            default: accept;
        }
    }

    state ipv6 {
        packet.extract(hdr.ipv6);
        transition accept;
    }
}

/********************************************************************************
 * Main
 */
control TopPipe(inout Parsed_packet hdr,
                inout user_metadata_t meta,
                inout digest_data_t digest_data,
                inout sume_metadata_t sume_metadata) {

    action do_nothing() {
        EthAddr_t temp = p.ethernet.dstAddr;
    }

    action send_to_port1() {
        sume_metadata.dst_port = 1;
    }

    action modify_ipv6 () {
        hdr.ipv6.src_addr += 1;
        hdr.ipv6.dst_addr += 2;

        send_to_port1();
    }

    table lookup_table {
        key = {
            p.ethernet.dstAddr: exact;
        }

        actions = {
            do_nothing;
            send_to_port1;
        }
        size = 64;
        default_action = modify_ipv6;
    }

    apply {
        lookup_table.apply();
    }
}

/********************************************************************************
 * Deparser
 */

@Xilinx_MaxPacketRegion(1024)
control TopDeparser(packet_out b,
                    in Parsed_packet p,
                    in user_metadata_t user_metadata,
                    inout digest_data_t digest_data,
                    inout sume_metadata_t sume_metadata) {
    apply {
        b.emit(p.ethernet);
    }
}

/********************************************************************************
 * Switch
 */

SimpleSumeSwitch(
    TopParser(),
    TopPipe(),
    TopDeparser()
) main;
